import unittest2 as unittest
from chembl_webresource_client.scripts.utils import resolve
from chembl_webresource_client.scripts.utils import sdf_split
from chembl_webresource_client.scripts.utils import get_parents
from chembl_webresource_client.scripts.utils import resolve_target
from chembl_webresource_client.scripts.utils import get_serializer
from chembl_webresource_client.scripts.utils import inspect_synonyms
from chembl_webresource_client.scripts.utils import convert_to_smiles
from chembl_webresource_client.scripts.utils import mols_to_targets
from chembl_webresource_client.scripts.utils import targets_to_mols
from chembl_webresource_client.scripts.utils import enhance_mols_with_parents
from chembl_webresource_client.scripts.utils import inchi_key_regex, chembl_id_regex, smiles_regex
from chembl_webresource_client.utils import utils
from chembl_webresource_client.elastic_client import ElasticClient
from io import StringIO


sdf = '''
  SciTegic12231509382D

 31 35  0  0  0  0            999 V2000
   11.2581  -12.0726    0.0000 F   0  0
    9.1585   -8.7448    0.0000 C   0  0
   11.1775   -9.6257    0.0000 C   0  0
   14.0880  -12.8949    0.0000 O   0  0
    9.1518  -10.1649    0.0000 N   0  0
   13.3806  -11.6670    0.0000 C   0  0
   11.2640  -10.4378    0.0000 N   0  0
   10.7879   -8.7414    0.0000 H   0  0
   14.8024  -10.8442    0.0000 C   0  0
   12.6734  -10.4371    0.0000 C   0  0
    9.9705  -10.1649    0.0000 C   0  0  1  0  0  0
   14.8024  -11.6691    0.0000 C   0  0
   12.6752  -12.0736    0.0000 C   0  0
   14.0905  -10.4284    0.0000 N   0  0
   10.3841   -9.4526    0.0000 C   0  0  1  0  0  0
    9.9789   -8.7448    0.0000 C   0  0
   16.2201  -11.6726    0.0000 O   0  0
    9.5544  -10.8666    0.0000 H   0  0
    8.7449   -9.4473    0.0000 C   0  0
   11.9594   -9.2141    0.0000 C   0  0
   12.6734   -9.6204    0.0000 O   0  0
   13.6814   -8.9080    0.0000 C   0  0
   11.9657  -11.6649    0.0000 C   0  0
   14.0880  -12.0781    0.0000 C   0  0
   11.9657  -10.8458    0.0000 C   0  0
   14.0905   -9.6117    0.0000 C   0  0
   14.4982   -8.9080    0.0000 C   0  0
   15.5127  -12.8957    0.0000 O   0  0
   13.3806  -10.8422    0.0000 C   0  0
   10.5186  -10.7695    0.0000 C   0  0
   15.5127  -12.0789    0.0000 C   0  0
 10 21  1  0
 12 31  1  0
  9 14  1  0
 29 14  1  0
 14 26  1  0
 11 30  1  0
 15 16  1  0
 21 20  1  0
 13  6  2  0
 11 18  1  6
 25 23  2  0
 27 26  1  0
 31 28  1  0
 24  4  2  0
 31 17  2  0
 26 22  1  0
 16  2  1  0
 23 13  1  0
  3 15  1  0
 25  7  1  0
  2 19  1  0
 22 27  1  0
 19  5  1  0
 29 10  2  0
 12  9  2  0
 10 25  1  0
 30  7  1  0
  6 24  1  0
 24 12  1  0
 15  8  1  6
  7  3  1  0
 11 15  1  0
 11  5  1  0
 23  1  1  0
 29  6  1  0
M  END
> <chembl_id>
CHEMBL32


$$$$

  SciTegic01161211212D

 32 35  0  0  0  0            999 V2000
    2.7125   -2.8375    0.0000 N   0  0
    3.3292   -3.9000    0.0000 C   0  0
    2.1042   -3.1917    0.0000 C   0  0
    2.1042   -3.9000    0.0000 C   0  0
    0.2542   -2.8667    0.0000 N   0  0
    3.3292   -3.1917    0.0000 C   0  0
    2.7125   -4.2417    0.0000 C   0  0
    1.4792   -2.8375    0.0000 C   0  0
    0.8667   -3.1917    0.0000 C   0  0
    0.8667   -3.9000    0.0000 C   0  0
    2.7125   -2.1292    0.0000 C   0  0
    1.4792   -4.2417    0.0000 C   0  0
    3.9500   -4.2417    0.0000 C   0  0
    0.1375   -2.1542    0.0000 C   0  0
   -0.4000   -3.1500    0.0000 C   0  0
   -0.5208   -2.0125    0.0000 C   0  0  1  0  0  0
   -0.8708   -2.6167    0.0000 C   0  0  2  0  0  0
    3.0917   -1.4542    0.0000 C   0  0
    2.3542   -1.4542    0.0000 C   0  0
   -1.5833   -2.6042    0.0000 N   0  0
    2.7125   -4.9417    0.0000 O   0  0
    3.9500   -4.9500    0.0000 O   0  0
    1.4792   -2.1292    0.0000 O   0  0
    0.2542   -4.2375    0.0000 F   0  0
    4.5542   -3.9000    0.0000 O   0  0
    4.3500   -2.6167    0.0000 Cl  0  0
   -0.8583   -1.3875    0.0000 C   0  0
   -1.9333   -2.0125    0.0000 C   0  0
    0.8875   -1.7792    0.0000 C   0  0
   -1.5708   -1.3875    0.0000 C   0  0
   -1.2333   -3.2292    0.0000 H   0  0
   -0.1708   -1.3917    0.0000 H   0  0
  2  6  2  0
  3  1  1  0
  4  3  2  0
  5  9  1  0
  6  1  1  0
  7  2  1  0
  8  3  1  0
  9  8  2  0
 10  9  1  0
 11  1  1  0
 12  4  1  0
 13  2  1  0
 14  5  1  0
 15  5  1  0
 16 14  1  0
 17 15  1  0
 18 11  1  0
 19 11  1  0
 20 17  1  0
 21  7  2  0
 22 13  2  0
 23  8  1  0
 24 10  1  0
 25 13  1  0
 27 16  1  0
 28 20  1  0
 29 23  1  0
 30 27  1  0
 17 31  1  6
 16 32  1  6
 19 18  1  0
  7  4  1  0
 10 12  2  0
 17 16  1  0
 28 30  1  0
M  END
> <chembl_id>
CHEMBL1200735


$$$$
'''

class TestSequenceFunctions(unittest.TestCase):

    def test_resolve(self):

        ret = resolve('viagra')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL1737')

        ret = resolve('gleevec')
        self.assertEqual(len(ret), 2)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL941')
        self.assertEqual(ret[1]['molecule_chembl_id'], 'CHEMBL1642')

        ret = resolve('gleevec', single_result=True)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL941')

        ret = resolve('aspirin')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL25')

        ret = resolve('chembl59')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL59')

        ret = resolve('OLEOYL')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL191896')

        ret = resolve('Fc1ccc2[nH]c3CCN(CCc4ccncc4)Cc3c2c1')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL300443')

        ret = resolve('VYFYYTLLBUKUHU-WTJCDBBSSA-N')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL3247442')

        ret = resolve('Vigamox')
        self.assertEqual(len(ret), 2)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL32')
        self.assertEqual(ret[1]['molecule_chembl_id'], 'CHEMBL1200735')

        ret = resolve('Vigamox', single_result=True)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL32')

        ret = resolve('InChI=1S/C8H12N4O5/c9-6(16)7-10-2-12(11-7)8-5(15)4(14)3(1-13)17-8/h2-5,8,13-15H,1H2,(H2,9,16)')
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]['molecule_chembl_id'], 'CHEMBL34')

    def test_resolve_target(self):
        self.assertEqual(len(resolve_target('5-HT')), 5)
        res = resolve_target('Serotonin transporter')
        self.assertEqual(len(res), 4)
        self.assertTrue(all([r['pref_name'] == 'Serotonin transporter' for r in res]))
        res = resolve_target('Dopamine D1 receptor')
        self.assertEqual(len(res), 6)
        self.assertTrue(all([r['pref_name'] == 'Dopamine D1 receptor' for r in res]))
        res = resolve_target('Influenza A virus')
        self.assertEqual(len(res), 1)
        self.assertTrue(all([r['pref_name'] == 'Influenza A virus' for r in res]))
        res = resolve_target('HERG')
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]['target_chembl_id'], 'CHEMBL240')
        res = resolve_target('pde5')
        self.assertEqual(len(res), 10)
        self.assertTrue(all(
            ['pdbe5' in r['pref_name'].lower() or 'phosphodiesterase' in r['pref_name'].lower() for r in res]))
        res = resolve_target('BRD4')
        self.assertEqual(len(res), 2)
        self.assertTrue(all([r['pref_name'] == 'Bromodomain-containing protein 4' for r in res]))

    def test_chembl_id_serialiser(self):
        serialiser = get_serializer('chembl_id')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), 'Name:\tChEMBL ID:\n')
        self.assertEqual(serialiser.serialize_line(mols), 'CHEMBL32,CHEMBL1200735\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True), 'CHEMBL32,CHEMBL1200735\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'),
                         'Vigamox\tCHEMBL32,CHEMBL1200735\n')

    def test_smiles_serialiser(self):
        serialiser = get_serializer('smi')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), 'SMILES Name\n')
        self.assertEqual(serialiser.serialize_line(mols),
                         'COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O\n'
                         'Cl.COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True),
                         'COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O\n'
                         'Cl.COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'),
                         'COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O Vigamox\n'
                         'Cl.COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O Vigamox\n')

    def test_sdf_serialiser(self):

        serialiser = get_serializer('sdf')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), '')
        self.assertEqual(serialiser.serialize_line(mols), sdf, serialiser.serialize_line(mols))
        self.assertEqual(serialiser.serialize_line(mols, human=True), sdf)
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'), sdf)
        serialiser = get_serializer('mol')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), '')
        self.assertEqual(serialiser.serialize_line(mols), sdf, serialiser.serialize_line(mols))
        self.assertEqual(serialiser.serialize_line(mols, human=True), sdf)
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'), sdf)

    def test_inchi_serialiser(self):
        serialiser = get_serializer('inchi')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), 'Name:\tInChI:\n')
        self.assertEqual(serialiser.serialize_line(mols),
                         'InChI=1S/C21H24FN3O4/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11'
                         '-3-2-6-23-16(11)10-24/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28)/t11-,16+/m0/s1\tInChI=1S/C21'
                         'H24FN3O4.ClH/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11-3-2-6-2'
                         '3-16(11)10-24;/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28);1H/t11-,16+;/m0./s1\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True),
                         'InChI=1S/C21H24FN3O4/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11'
                         '-3-2-6-23-16(11)10-24/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28)/t11-,16+/m0/s1\tInChI=1S/C21'
                         'H24FN3O4.ClH/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11-3-2-6-2'
                         '3-16(11)10-24;/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28);1H/t11-,16+;/m0./s1\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'),
                         'Vigamox\tInChI=1S/C21H24FN3O4/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(2'
                         '0)24-8-11-3-2-6-23-16(11)10-24/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28)/t11-,16+/m0/s1\tInC'
                         'hI=1S/C21H24FN3O4.ClH/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-1'
                         '1-3-2-6-23-16(11)10-24;/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28);1H/t11-,16+;/m0./s1\n')

    def test_chembl_key_serialiser(self):
        serialiser = get_serializer('inchi_key')
        mols = resolve('Vigamox')
        buf = StringIO()
        serialiser.write_header(buf)
        buf.seek(0)
        self.assertEqual(buf.read(), 'Name:\tInChI Key:\n')
        self.assertEqual(serialiser.serialize_line(mols), 'FABPRXSRWADJSP-MEDUHNTESA-N,IDIIJJHBXUESQI-DFIJPDEKSA-N\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True),
                         'FABPRXSRWADJSP-MEDUHNTESA-N,IDIIJJHBXUESQI-DFIJPDEKSA-N\n')
        self.assertEqual(serialiser.serialize_line(mols, human=True, name='Vigamox'),
                         'Vigamox\tFABPRXSRWADJSP-MEDUHNTESA-N,IDIIJJHBXUESQI-DFIJPDEKSA-N\n')

    def test_get_parents(self):
        parents = get_parents(resolve('CHEMBL1642'))
        self.assertEqual(len(parents), 1)
        self.assertEqual(parents[0]['molecule_chembl_id'], 'CHEMBL941')

        parents = get_parents(resolve('CHEMBL941'))
        self.assertEqual(len(parents), 1)
        self.assertEqual(parents[0]['molecule_chembl_id'], 'CHEMBL941')

        parents = get_parents(resolve('CHEMBL1200735'))
        self.assertEqual(len(parents), 1)
        self.assertEqual(parents[0]['molecule_chembl_id'], 'CHEMBL32')

        parents = get_parents(resolve('CHEMBL32'))
        self.assertEqual(len(parents), 1)
        self.assertEqual(parents[0]['molecule_chembl_id'], 'CHEMBL32')

        parents = get_parents(resolve('CHEMBL1642') + resolve('CHEMBL1200735'))
        self.assertEqual(len(parents), 2)
        self.assertEqual(parents[1]['molecule_chembl_id'], 'CHEMBL941')
        self.assertEqual(parents[0]['molecule_chembl_id'], 'CHEMBL32')

    def test_enhance_mols_with_parents(self):
        enhanced = [x['molecule_chembl_id'] for x in enhance_mols_with_parents(resolve('CHEMBL1642'))]
        self.assertEqual(enhanced, ['CHEMBL941', 'CHEMBL1642'])

    def test_regexes(self):
        self.assertTrue(inchi_key_regex.match('FABPRXSRWADJSP-MEDUHNTESA-N'))
        self.assertFalse(inchi_key_regex.match('FAPRXSRWADJSP-MEDUHNTESA-N'))
        self.assertFalse(inchi_key_regex.match('FABPRXSRWADJSP-MEDUNTESA-N'))
        self.assertFalse(inchi_key_regex.match('FABPRXSRWADJSP-MEDUHNTESA-'))
        self.assertFalse(inchi_key_regex.match('FABPRXSRWADJSP\MEDUHNTESA-N'))
        self.assertFalse(inchi_key_regex.match('FABPRXSRWADJSP-MEDUHNTESA\\N'))

        self.assertTrue(chembl_id_regex.match('CHEMBL25'))
        self.assertFalse(chembl_id_regex.match('CHEML25'))
        self.assertFalse(chembl_id_regex.match('25'))
        self.assertFalse(chembl_id_regex.match('CHEMBL_25'))
        self.assertFalse(chembl_id_regex.match('CHEMBL'))

        self.assertTrue(smiles_regex.match('Fc1ccc2[nH]c3CCN(CCc4ccncc4)Cc3c2c1'))
        self.assertTrue(smiles_regex.match('COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O'))
        self.assertTrue(
            smiles_regex.match('Cl.COc1c(N2C[C@@H]3CCCN[C@@H]3C2)c(F)cc4C(=O)C(=CN(C5CC5)c14)C(=O)O'))
        self.assertTrue(smiles_regex.match('cc'))

    def test_inspect_synonyms(self):
        dopamine = resolve('dopamine')[0]
        self.assertTrue(inspect_synonyms(dopamine, 'Intropin'))
        self.assertTrue(inspect_synonyms(dopamine, 'parcopa'))
        self.assertTrue(inspect_synonyms(dopamine, 'SINEMET'))
        self.assertFalse(inspect_synonyms(dopamine, 'viagra'))
        self.assertFalse(inspect_synonyms(dopamine, ''))

    def test_sdf_split(self):
        count = 0
        inchis = ['InChI=1S/C21H24FN3O4/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11-3-2-6-'
                  '23-16(11)10-24/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28)/t11-,16+/m0/s1',
                  'InChI=1S/C21H24FN3O4.ClH/c1-29-20-17-13(19(26)14(21(27)28)9-25(17)12-4-5-12)7-15(22)18(20)24-8-11-3-'
                  '2-6-23-16(11)10-24;/h7,9,11-12,16,23H,2-6,8,10H2,1H3,(H,27,28);1H/t11-,16+;/m0./s1']
        f = StringIO(sdf)
        for idx, mol in enumerate(sdf_split(f)):
            count += 1
            self.assertEqual(utils.ctab2inchi(mol), inchis[idx])
        self.assertEqual(count, 2)

    def test_convert_to_smiles(self):
        f = StringIO(sdf)
        inp = convert_to_smiles(f)
        self.assertEqual(inp.read(), 'COc1c(N2CC3CCCNC3C2)c(F)cc2c(=O)c(C(=O)O)cn(C3CC3)c12 0\n'
                                     'COc1c(N2CC3CCCNC3C2)c(F)cc2c(=O)c(C(=O)O)cn(C3CC3)c12.Cl 1\n')

    def test_combine_sdf_split_with_smiles_conversion(self):
        smiles = ['COc1c(N2CC3CCCNC3C2)c(F)cc2c(=O)c(C(=O)O)cn(C3CC3)c12',
                  'COc1c(N2CC3CCCNC3C2)c(F)cc2c(=O)c(C(=O)O)cn(C3CC3)c12.Cl']
        f = StringIO(sdf)
        for idx, mol in enumerate(sdf_split(f)):
            self.assertEqual(utils.ctab2smiles(mol).split()[2], smiles[idx])

    def test_mols_to_targets(self):
        ret = mols_to_targets(['CHEMBL819'], organism='Escherichia coli')
        serialiser = get_serializer('uniprot')
        self.assertEqual(set(serialiser.serialize_line(ret).strip().split(',')), {'A1E3K9', 'P62593', 'P35695'})

        ret = mols_to_targets(['CHEMBL819'], organism='Escherichia coli', include_parents=True)
        serialiser = get_serializer('uniprot')
        self.assertEqual(set(serialiser.serialize_line(ret).strip().split(',')),
                         {'A1E3K9', 'P62593', 'P35695', 'P00811'})

        ret = mols_to_targets(['CHEMBL25', 'CHEMBL1'])
        serialiser = get_serializer('gene_name')
        self.assertTrue({'POLK', 'ADRA2RL2', 'CCR2', 'CHRM5', 'ADRA2L1', 'CMKAR1', 'PTPRC', 'HADH2', 'ITGAB', 'VIPR1'
                         }.issubset(set(serialiser.serialize_line(ret).strip().split(','))))

        ret = mols_to_targets(['CHEMBL2', 'CHEMBL1737'])
        serialiser = get_serializer('chembl_id')
        targets_a = set(serialiser.serialize_line(ret).strip().split(','))
        self.assertTrue({'CHEMBL213', 'CHEMBL1916', 'CHEMBL205', 'CHEMBL4071', 'CHEMBL1909043', 'CHEMBL2622',
                         }.issubset(targets_a))

        ret = mols_to_targets(['CHEMBL2', 'CHEMBL1737'], only_ids=True)
        serialiser = get_serializer('chembl_id')
        targets_b = set(serialiser.serialize_line(ret).strip().split(','))
        self.assertTrue({'CHEMBL213', 'CHEMBL1916', 'CHEMBL205', 'CHEMBL4071', 'CHEMBL1909043', 'CHEMBL2622',
                         }.issubset(targets_b))

        self.assertEqual(targets_a, targets_b)

        drugs = ['Viagra', 'Gleevec']

        ret = mols_to_targets([resolve(x)[0]['molecule_chembl_id'] for x in drugs])
        serialiser = get_serializer('uniprot')
        self.assertTrue({'Q9BQI3', 'P00523', 'O15111', 'P22612', 'P25021', 'P16591', 'Q13153', 'Q8WXR4', 'P28564',
                         }.issubset(set(serialiser.serialize_line(ret).strip().split(','))))

    def test_targets_to_mols(self):

        ret = targets_to_mols([resolve_target(x)[0]['target_chembl_id'] for x in ['POLK']])
        serialiser = get_serializer('chembl_id')
        self.assertTrue({'CHEMBL6', 'CHEMBL25', 'CHEMBL10', 'CHEMBL30', 'CHEMBL50', 'CHEMBL71', 'CHEMBL28', 'CHEMBL66'
                          }.issubset(set(serialiser.serialize_line(ret).strip().split(','))))

        ret = targets_to_mols([resolve_target(x)[0]['target_chembl_id'] for x in ['HERG']])
        serialiser = get_serializer('smiles')
        self.assertTrue({'CN1C(=O)N(CC(=O)c2ccccc2)C(=O)c3c1nc(N4CCC[C@H](N)C4)n3CC=C(C)C',
                         'C[C@]12CCC(=O)C=C1CC[C@@H]3[C@@H]2CC[C@@]4(C)[C@H]3CC[C@@]4(O)C#C',
                         'CS(=O)(=O)N1CCC(CN([C@@H]2CC[C@@]3(CC3C2)c4cccc(c4)C#N)C(=O)Nc5ccc(F)c(F)c5)CC1',
                         'O=S(=O)(NCCCN1CCN(CC1)c2nsc3ccccc23)c4cccc5scnc45',
                         'Cl.CC(C)n1nc(C(=O)NCC2CCN(CCNS(=O)(=O)C)CC2)c3ccccc13',
                         'CC1CN(CC(=O)N[C@@H]2C3CC4CC2C[C@@](C4)(C3)C(=O)N)S(=O)(=O)N(C1)c5c(Cl)cc(Cl)cc5Cl',
                         'Oc1cccc2CC(CN3CCC4(CC3)CCc5ccccc45)NCc12',
                         '[O-][N+](=O)c1ccc(cc1)c2nc(c3ccc(F)cc3)c([nH]2)c4ccncc4'
                         }.issubset(set(serialiser.serialize_line(ret).strip().split())))

        ret = targets_to_mols([resolve_target(x)[0]['target_chembl_id'] for x in ['CMKAR1']])
        serialiser = get_serializer('inchi')
        self.assertTrue({
                            'InChI=1S/C7H6O2/c8-7(9)6-4-2-1-3-5-6/h1-5H,(H,8,9)',
                            'InChI=1S/C11H7NS/c13-8-12-11-7-3-5-9-4-1-2-6-10(9)11/h1-7H',
                            'InChI=1S/CHCl3/c2-1(3)4/h1H',
                            'InChI=1S/H4N2/c1-2/h1-2H2',
                            'InChI=1S/C8H8N4/c9-11-8-7-4-2-1-3-6(7)5-10-12-8/h1-5H,9H2,(H,11,12)',
                            'InChI=1S/CH4N2O2/c2-1(4)3-5/h5H,(H3,2,3,4)',
                            'InChI=1S/CH3O5P/c2-1(3)7(4,5)6/h(H,2,3)(H2,4,5,6)',
                            'InChI=1S/C4H11NO3/c5-4(1-6,2-7)3-8/h6-8H,1-3,5H2',
                            'InChI=1S/C9H9Cl2N3O/c10-6-2-1-3-7(11)5(6)4-8(15)14-9(12)13/h1-3H,4H2,(H4,12,13,14,15)',
                        }.issubset(set(serialiser.serialize_line(ret).strip().split('\t'))))

        ret = targets_to_mols([resolve_target(x)[0]['target_chembl_id'] for x in ['ADRA2L1', 'VIPR1']])
        serialiser = get_serializer('inchi_key')
        self.assertTrue({
                            'OELFLUMRDSZNSF-BRWVUGGUSA-N',
                            'KJPRLNWUNMBNBZ-QPJJXVBHSA-N',
                            'VEPKQEUBKLEPRA-UHFFFAOYSA-N',
                            'ACGUYXCXAPNIKK-UHFFFAOYSA-N',
                            'MVGSNCBCUWPVDA-MFOYZWKCSA-N',
                            'JZHXLYHEWUWHQL-LVYIWIAJSA-N',
                            'CUVBGWMAORETGV-UHFFFAOYSA-N',
                            'DVSMVUMYJDOPJQ-UHFFFAOYSA-N',
                         }.issubset(set(serialiser.serialize_line(ret).strip().split(','))))

    def test_elastic(self):
        es = ElasticClient()
        ret = es.search_target('CMKAR1')
        self.assertEqual(ret, ['CHEMBL4029', 'CHEMBL2096909'])

        ret = es.search_molecule('viagra')
        self.assertEqual(ret, ['CHEMBL1737'])

        ret = es.search_assay('modulation activity')
        self.assertEqual(ret, ['CHEMBL3371687'])

        ret = es.search_cell_line('caco-2')
        self.assertEqual(ret, ['CHEMBL3307519'])

        ret = es.search_document('nowotarski')
        self.assertEqual(ret, ['CHEMBL3400002'])

        ret = es.search_tissue('liver')
        self.assertEqual(ret, ['CHEMBL3559723'])

if __name__ == '__main__':
    unittest.main()


from chembl_webresource_client.settings import Settings

Settings.Instance().MAX_LIMIT = 500

import unittest2 as unittest
import json
from chembl_webresource_client.new_client import new_client
from chembl_webresource_client.utils import utils
from chembl_webresource_client.unichem import unichem_client as unichem
from statistics import median


class TestDocsExamples(unittest.TestCase):

    def test_search_molecule_by_synonym(self):
        molecule = new_client.molecule
        molecule.set_format('json')
        res = molecule.search('viagra')
        self.assertEqual(len(res), 2)
        self.assertTrue(
            all([any(['Sildenafil' in y['molecule_synonym'] for y in x['molecule_synonyms']]) for x in res]))

    def test_search_target_by_gene_name(self):
        target = new_client.target
        gene_name = 'BRD4'
        res = target.search(gene_name)
        self.assertEqual(len(res), 2)
        self.assertTrue(all([all(
            [any(['brd4' in z['component_synonym'].lower() for z in y['target_component_synonyms']]) for y in
             x['target_components']]) for x in res]))

    def test_search_gene_name_in_target_synonym(self):
        target = new_client.target
        gene_name = 'GABRB2'
        res = target.filter(target_synonym__icontains=gene_name)
        self.assertEqual(len(res), 14)
        self.assertTrue(all([any(
            [any(['GABRB2' in z['component_synonym'].upper() for z in y['target_component_synonyms']]) for y in
             x['target_components']]) for x in res]))

    def test_chembl_id_to_uniprot(self):
        chembl_ids = ['CHEMBL819', 'CHEMBL820', 'CHEMBL821']
        compounds2targets = dict()

        # First, let's just parse the csv file to extract compounds ChEMBL IDs:
        for chembl_id in chembl_ids:
            compounds2targets[chembl_id] = set()

        # OK, we have our source IDs, let's process them in chunks:
        chunk_size = 50
        keys = list(compounds2targets.keys())

        for i in range(0, len(keys), chunk_size):
            # we jump from compounds to targets through activities:
            activities = new_client.activity.filter(molecule_chembl_id__in=keys[i:i + chunk_size]).only(
                ['molecule_chembl_id', 'target_chembl_id'])
            # extracting target ChEMBL IDs from activities:
            for act in activities:
                compounds2targets[act['molecule_chembl_id']].add(act['target_chembl_id'])

        # OK, now our dictionary maps from compound ChEMBL IDs into target ChEMBL IDs
        # We would like to replace target ChEMBL IDs with uniprot IDs

        for key, val in compounds2targets.items():
            # We don't know how many targets are assigned to a given compound so again it's
            # better to process targets in chunks:
            lval = list(val)
            uniprots = set()
            for i in range(0, len(val), chunk_size):
                targets = new_client.target.filter(target_chembl_id__in=lval[i:i + chunk_size]).only(
                    ['target_components'])
                uniprots |= set(
                    sum([[comp['accession'] for comp in t['target_components']] for t in targets], []))
            compounds2targets[key] = uniprots

        self.assertTrue({'A1E3K9', 'P35695', 'P62593'}.issubset(compounds2targets['CHEMBL819']))
        self.assertTrue('P00811' not in compounds2targets['CHEMBL819'])
        self.assertTrue(len(compounds2targets['CHEMBL819']) == 14)
        self.assertTrue(len(compounds2targets['CHEMBL820']) == 123)
        self.assertTrue(len(compounds2targets['CHEMBL821']) == 13)

    def test_chembl_id_to_uniprot_with_parents(self):
        chembl_ids = ['CHEMBL819', 'CHEMBL820', 'CHEMBL821']
        organism = 'Escherichia coli'
        compounds2targets = dict()
        header = True

        for chembl_id in chembl_ids:
            compounds2targets[chembl_id] = set()

        chunk_size = 50
        keys = list(compounds2targets.keys())

        ID_forms = dict()
        for x in keys:
            ID_forms[x] = set()

        for i in range(0, len(keys), chunk_size):
            for form in new_client.molecule_form.filter(parent_chembl_id__in=keys[i:i + chunk_size]):
                ID_forms[form['parent_chembl_id']].add(form['molecule_chembl_id'])

        for i in range(0, len(keys), chunk_size):
            for form in new_client.molecule_form.filter(molecule_chembl_id__in=keys[i:i + chunk_size]):
                ID_forms[form['molecule_chembl_id']].add(form['parent_chembl_id'])

        values = []
        for x in ID_forms.values():
            values.extend(x)
        forms_to_ID = dict()
        for x in values:
            forms_to_ID[x] = set()

        for k in forms_to_ID:
            for parent, molecule in ID_forms.items():
                if k in molecule:
                    forms_to_ID[k] = parent

        for i in range(0, len(values), chunk_size):
            activities = new_client.activity.filter(molecule_chembl_id__in=values[i:i + chunk_size]).filter(
                target_organism__istartswith=organism).only(['molecule_chembl_id', 'target_chembl_id'])
            for act in activities:
                compounds2targets[forms_to_ID[act['molecule_chembl_id']]].add(act['target_chembl_id'])

        for key, val in compounds2targets.items():
            lval = list(val)
            uniprots = set()
            for i in range(0, len(val), chunk_size):
                targets = new_client.target.filter(target_chembl_id__in=lval[i:i + chunk_size]).only(
                    ['target_components'])
                uniprots = uniprots.union(
                    set(sum([[comp['accession'] for comp in t['target_components']] for t in targets], [])))
            compounds2targets[key] = uniprots

        self.assertTrue('P00811' in compounds2targets['CHEMBL819'])
        self.assertTrue(len(compounds2targets['CHEMBL819']) == 4)
        self.assertTrue(len(compounds2targets['CHEMBL820']) == 0)
        self.assertTrue(len(compounds2targets['CHEMBL821']) == 0)

    def test_chembl_id_to_human_gene_names(self):
        chembl_ids = ['CHEMBL819', 'CHEMBL820', 'CHEMBL821']
        compounds2targets = dict()

        # First, let's just parse the csv file to extract compounds ChEMBL IDs:
        for chembl_id in chembl_ids:
            compounds2targets[chembl_id] = set()

        # OK, we have our source IDs, let's process them in chunks:
        chunk_size = 50
        keys = list(compounds2targets.keys())

        for i in range(0, len(keys), chunk_size):
            # we jump from compounds to targets through activities:
            activities = new_client.activity.filter(molecule_chembl_id__in=keys[i:i + chunk_size]).only(
                ['molecule_chembl_id', 'target_chembl_id'])
            # extracting target ChEMBL IDs from activities:
            for act in activities:
                compounds2targets[act['molecule_chembl_id']].add(act['target_chembl_id'])

        # OK, now our dictionary maps from compound ChEMBL IDs into target ChEMBL IDs
        # We would like to replace target ChEMBL IDs with uniprot IDs

        for key, val in compounds2targets.items():
            # We don't know how many targets are assigned to a given compound so again it's
            # better to process targets in chunks:
            lval = list(val)
            genes = set()
            for i in range(0, len(val), chunk_size):
                targets = new_client.target.filter(target_chembl_id__in=lval[i:i + chunk_size]).only(
                    ['target_components'])
                for target in targets:
                    for component in target['target_components']:
                        for synonym in component['target_component_synonyms']:
                            if synonym['syn_type'] == "GENE_SYMBOL":
                                genes.add(synonym['component_synonym'])
            compounds2targets[key] = genes

        self.assertTrue({'ORF35', 'Pept2', 'Oat1'}.issubset(compounds2targets['CHEMBL819']),
                        compounds2targets['CHEMBL819'])
        self.assertTrue({'B2AR', 'AVPR1', 'UBP41'}.issubset(compounds2targets['CHEMBL820']),
                        compounds2targets['CHEMBL820'])
        self.assertTrue({'OATP1B3', 'OAT3', 'OCT2'}.issubset(compounds2targets['CHEMBL821']),
                        compounds2targets['CHEMBL821'])

    def test_similarity_85(self):

        similarity = new_client.similarity
        res = similarity.filter(smiles="CO[C@@H](CCC#C\C=C/CCCC(C)CCCCC=C)C(=O)[O-]", similarity=85)
        self.assertTrue(len(res) >= 2)
        self.assertTrue(all([float(r['similarity']) >= 80.0 for r in res]))
        self.assertTrue({'CHEMBL478779', 'CHEMBL477888'}.issubset(set([r['molecule_chembl_id'] for r in res])))

    def test_similarity_70(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        similarity = new_client.similarity
        aspirin_chembl_id = molecule.search('aspirin')[0]['molecule_chembl_id']
        res = similarity.filter(chembl_id=aspirin_chembl_id, similarity=70)
        self.assertTrue(len(res) >= 6)
        self.assertTrue(res[0]['similarity'] == '100')
        self.assertTrue(res[0]['molecule_chembl_id'] == 'CHEMBL25')
        self.assertTrue({'CHEMBL163148', 'CHEMBL351485', 'CHEMBL2260706', 'CHEMBL1234172', 'CHEMBL1359634'}.issubset(
            set([r['molecule_chembl_id'] for r in res])))

    def test_smiles_substructure(self):

        substructure = new_client.substructure
        res = substructure.filter(smiles="CN(CCCN)c1cccc2ccccc12")
        self.assertTrue(len(res) > 80)
        self.assertTrue({'CHEMBL442138', 'CHEMBL287955', 'CHEMBL38899', 'CHEMBL53821', 'CHEMBL55826'}.issubset(
            set([r['molecule_chembl_id'] for r in res])))

    def test_chembl_id_substructure(self):

        substructure = new_client.substructure
        res = substructure.filter(chembl_id="CHEMBL25")
        self.assertTrue(len(res) > 380)
        self.assertTrue({'CHEMBL25', 'CHEMBL7666', 'CHEMBL10222', 'CHEMBL10008'}.issubset(
            set([r['molecule_chembl_id'] for r in res])))

    def test_get_by_chembl_id(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        m1 = molecule.get('CHEMBL25')
        self.assertEqual(m1['pref_name'], 'ASPIRIN')
        self.assertEqual(m1['molecule_chembl_id'], 'CHEMBL25')

    def test_get_by_smiles(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        m1 = molecule.get('CC(=O)Oc1ccccc1C(=O)O')
        self.assertEqual(m1['pref_name'], 'ASPIRIN')
        self.assertEqual(m1['molecule_chembl_id'], 'CHEMBL25')

    def test_get_by_smiles_flexmatch(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        res = molecule.filter(molecule_structures__canonical_smiles__flexmatch='CN(C)C(=N)N=C(N)N')
        self.assertEqual(len(res), 6)  # this returns 6 compounds
        self.assertTrue('CHEMBL1431' in [x['molecule_chembl_id'] for x in res])
        self.assertTrue('METFORMIN' in [x['pref_name'] for x in res])

    def test_get_by_inchi_key(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        m1 = molecule.get('BSYNRYMUTXBXSQ-UHFFFAOYSA-N')
        self.assertEqual(m1['pref_name'], 'ASPIRIN')
        self.assertEqual(m1['molecule_chembl_id'], 'CHEMBL25')

    def test_get_multiple_by_chmbl_ids(self):

        mols = ['CHEMBL6498', 'CHEMBL6499', 'CHEMBL6505']
        molecule = new_client.molecule
        molecule.set_format('json')
        records = molecule.get(['CHEMBL6498', 'CHEMBL6499', 'CHEMBL6505'])
        self.assertEqual(len(records), 3)
        self.assertTrue(len(set([x['molecule_chembl_id'] for x in records]) ^ set(mols)) == 0)

    def test_get_multiple_by_smiles(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        records = molecule.get(['CNC(=O)c1ccc(cc1)N(CC#C)Cc2ccc3nc(C)nc(O)c3c2',
                                'Cc1cc2SC(C)(C)CC(C)(C)c2cc1\\N=C(/S)\\Nc3ccc(cc3)S(=O)(=O)N',
                                'CC(C)C[C@H](NC(=O)[C@@H](NC(=O)[C@H](Cc1c[nH]c2ccccc12)NC'
                                '(=O)[C@H]3CCCN3C(=O)C(CCCCN)CCCCN)C(C)(C)C)C(=O)O'])
        self.assertEqual(len(records), 3)
        self.assertTrue(
            len(set([x['molecule_chembl_id'] for x in records]) ^ set(['CHEMBL6498', 'CHEMBL6499', 'CHEMBL6505'])) == 0)

    def test_get_multiple_by_inchi_keys(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        records = molecule.get(['XSQLHVPPXBBUPP-UHFFFAOYSA-N',
                                'JXHVRXRRSSBGPY-UHFFFAOYSA-N', 'TUHYVXGNMOGVMR-GASGPIRDSA-N'])
        self.assertEqual(len(records), 3)
        self.assertTrue(
            len(set([x['molecule_chembl_id'] for x in records]) ^ set(['CHEMBL6498', 'CHEMBL6499', 'CHEMBL6505'])) == 0)

    def test_pChembl(self):

        activities = new_client.activity
        res = activities.filter(molecule_chembl_id="CHEMBL25", pchembl_value__isnull=False)
        pchembls = [float(r['pchembl_value']) for r in res]
        self.assertTrue(5.2 < sum(pchembls) / len(pchembls) < 5.3)

    def test_get_pChembl_for_compound_and_target(self):

        activities = new_client.activity
        res = activities.filter(molecule_chembl_id="CHEMBL25", target_chembl_id="CHEMBL612545",
                                pchembl_value__isnull=False)
        pchembls = [float(r['pchembl_value']) for r in res]
        self.assertTrue(4.8 < sum(pchembls) / len(pchembls) < 4.9)

    def test_get_all_approved_drugs(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        approved_drugs = molecule.filter(max_phase=4)
        self.assertTrue(3900 < len(approved_drugs) < 4000)
        self.assertTrue(1.74 < sum(
            [float(d['molecule_properties']['alogp'] or 0) for d in approved_drugs if d['molecule_properties']]) / len(
            approved_drugs) < 1.76)

    def test_get_approved_drugs_for_lung_cancer(self):

        drug_indication = new_client.drug_indication
        molecule = new_client.molecule
        molecule.set_format('json')
        lung_cancer_ind = drug_indication.filter(efo_term__icontains="LUNG CARCINOMA")
        lung_cancer_mols = molecule.filter(
            molecule_chembl_id__in=[x['molecule_chembl_id'] for x in lung_cancer_ind])
        self.assertTrue(210 < len(lung_cancer_mols) < 220)
        self.assertTrue(
            set(['NICOTINE', 'SARACATINIB', 'BEMIPARIN']).issubset(set([l['pref_name'] for l in lung_cancer_mols])))

    def test_get_mols_with_no_ro5_violations(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        no_violations = molecule.filter(molecule_properties__num_ro5_violations=0)
        self.assertTrue(1200000 < len(no_violations) < 1300000)
        self.assertTrue(set(['GEMFIBROZIL', 'ANIROLAC', 'AZEPINDOLE']).issubset(
            set([n['pref_name'] for n in no_violations[:10000]])))

    def test_get_biotherapeutic_molecules(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        biotherapeutics = molecule.filter(biotherapeutic__isnull=False)
        self.assertTrue(21000 < len(biotherapeutics) < 22000)
        self.assertTrue(
            set(['LYPRESSIN', 'USTEKINUMAB', 'APICIDIN']).issubset(set([n['pref_name'] for n in biotherapeutics])))

    def test_get_natural_product_drugs(self):

        molecule = new_client.molecule
        molecule.set_format('sdf')
        natural_drugs = molecule.filter(natural_product=1)
        self.assertTrue(1900 < len(natural_drugs) < 2000)
        b'$$$$'.join([n for n in natural_drugs if n])
        molecule.set_format('json')

    def test_get_all_natural_products(self):

        document = new_client.document
        docs = document.filter(journal="J. Nat. Prod.").only('document_chembl_id')
        compound_record = new_client.compound_record
        records = compound_record.filter(
            document_chembl_id__in=[doc['document_chembl_id'] for doc in docs]).only(
            ['document_chembl_id', 'molecule_chembl_id'])
        molecule = new_client.molecule
        natural_products = molecule.filter(
            molecule_chembl_id__in=[rec['molecule_chembl_id'] for rec in records]).only(
            'molecule_structures')
        self.assertTrue(34000 < len(natural_products) < 35000)
        self.assertTrue(
            'CC(=O)Oc1ccccc1C(=O)O' in [d['molecule_structures']['canonical_smiles'] for d in natural_products if
                                        d['molecule_structures']])

    def test_get_light_molecules(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        light_molecules = molecule.filter(molecule_properties__mw_freebase__lte=300)
        self.assertTrue(310000 < len(light_molecules) < 320000)
        mean = sum([float(d['molecule_properties']['mw_freebase'] or 0) for d in light_molecules if
                    d['molecule_properties']]) / len(light_molecules)
        self.assertTrue(251 < mean < 252, mean)

    def test_get_light_molecules_ending_with_nib(self):

        molecule = new_client.molecule
        molecule.set_format('json')
        light_nib_molecules = molecule.filter(
            molecule_properties__mw_freebase__lte=300).filter(pref_name__iendswith="nib")
        self.assertEqual(len(light_nib_molecules), 1)
        self.assertEqual('SEMAXANIB', light_nib_molecules[0]['pref_name'])

    def test_get_ki_activities_for_herg(self):

        target = new_client.target
        activity = new_client.activity
        herg = target.search('herg')[0]
        herg_activities = activity.filter(target_chembl_id=herg['target_chembl_id']).filter(standard_type="Ki")
        self.assertTrue(2500 < len(herg_activities) < 3000)
        self.assertTrue(
            5300 < sum([float(x['standard_value'] or 0) for x in herg_activities]) / len(herg_activities) < 5400)

    def test_get_tg_gates_activities(self):

        activity = new_client.activity
        res = activity.search('"TG-GATES"')
        self.assertTrue(150000 < len(res) < 160000)
        mean = sum([float(r['pchembl_value'] or 0) for r in res]) / len(res)
        self.assertEqual(0.0, mean, mean)

    def test_get_b_or_f_type_activities(self):

        activity = new_client.activity
        res = activity.filter(target_chembl_id='CHEMBL3938', assay_type__iregex='(B|F)')
        self.assertTrue(400 < len(res) < 500)
        mean = sum([float(r['pchembl_value'] or 0) for r in res]) / len(res)
        self.assertTrue(0 < mean < 1, mean)

    def test_get_admet_related_inhibitor_assays(self):

        assay = new_client.assay
        res = assay.search('inhibitor').filter(assay_type='A')
        self.assertTrue(1000 < len(res) < 1100)
        self.assertEqual(median([r['confidence_score'] for r in res]), 1)

    def test_get_cell_by_cellosaurus_id(self):

        cell_line = new_client.cell_line
        res = cell_line.filter(cellosaurus_id="CVCL_0417")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]['cell_chembl_id'], 'CHEMBL3307686')

    def test_get_drugs_by_approval_year_and_name(self):

        drug = new_client.drug
        res = drug.filter(first_approval=1976).filter(usan_stem="-azosin")
        self.assertEqual(len(res), 1)
        self.assertTrue('Prazosin' in res[0]['synonyms'])

    def test_get_tissue_by_bto_id(self):

        tissue = new_client.tissue
        res = tissue.filter(bto_id="BTO:0001073")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]['tissue_chembl_id'], 'CHEMBL3638173')

    def test_get_tissue_by_caloha_id(self):

        tissue = new_client.tissue
        res = tissue.filter(caloha_id="TS-0490")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]['tissue_chembl_id'], 'CHEMBL3638176')

    def test_get_tissue_by_uberon_id(self):

        tissue = new_client.tissue
        res = tissue.filter(uberon_id="UBERON:0000173")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]['tissue_chembl_id'], 'CHEMBL3638177')

    def test_get_tissue_by_name(self):

        tissue = new_client.tissue
        res = tissue.filter(pref_name__istartswith='blood')
        self.assertTrue(len(res) >= 3)
        self.assertTrue(set(['Blood', 'Blood/Brain', 'Blood/Uterus']).issubset(set([r['pref_name'] for r in res])))

    def test_get_documents_for_cytokine(self):

        document = new_client.document
        res = document.search('cytokine')
        self.assertTrue(300 < len(res) < 400)
        self.assertEqual(median([x['year'] for x in res]), 2010)

    def test_search_compound_in_unichem(self):

        ret = unichem.get('AIN')
        self.assertEqual(len(ret), 1)
        self.assertTrue(set(['aspirin', 'CHEMBL25', 'SCHEMBL1353']).issubset(
            set([x['src_compound_id'] for x in ret[list(ret.keys())[0]]])))

    def test_resolve_inchi_key_to_inchi(self):

        ret = unichem.inchiFromKey('AAOVKJBEBIDNHE-UHFFFAOYSA-N')
        self.assertEqual(ret[0]['standardinchi'],
                         'InChI=1S/C16H13ClN2O/c1-19-14-8-7-12(17)9-13(14)16(18-10-15(19)20)11-5-3-2-4-6-11/h2-9H,10H2,1H3')

    def test_covert_smiles_to_ctab(self):

        aspirin = utils.smiles2ctab('O=C(Oc1ccccc1C(=O)O)C')
        self.assertTrue('V2000' in aspirin)

    def test_convert_smiles_to_image_and_back_to_smiles(self):

        aspirin = 'CC(=O)Oc1ccccc1C(=O)O'
        im = utils.smiles2image(aspirin)
        mol = utils.image2ctab(im)
        smiles = utils.ctab2smiles(mol).split()[2]
        self.assertEqual(smiles[-10:], aspirin[-10:])  # TODO: fix OSRA!

    def test_compute_fingerprints(self):

        aspirin = utils.smiles2ctab('O=C(Oc1ccccc1C(=O)O)C')
        fingerprints = utils.sdf2fps(aspirin)
        self.assertTrue(fingerprints.startswith('#FPS'))

    def test_compute_maximal_common_substructure(self):

        smiles = ["O=C(NCc1cc(OC)c(O)cc1)CCCC/C=C/C(C)C", "CC(C)CCCCCC(=O)NCC1=CC(=C(C=C1)O)OC", "c1(C=O)cc(OC)c(O)cc1"]
        mols = [utils.smiles2ctab(smile) for smile in smiles]
        sdf = ''.join(mols)
        result = utils.mcs(sdf)
        self.assertEqual(result, '[#6]1(-[#6]):[#6]:[#6](-[#8]-[#6]):[#6](:[#6]:[#6]:1)-[#8]')

    def test_compute_molecular_descriptors(self):

        aspirin = utils.smiles2ctab('O=C(Oc1ccccc1C(=O)O)C')

        num_atoms = json.loads(utils.getNumAtoms(aspirin))[0]
        self.assertEqual(num_atoms, 13)

        mol_wt = json.loads(utils.molWt(aspirin))[0]
        self.assertTrue(180 < mol_wt < 181)

        log_p = json.loads(utils.logP(aspirin))[0]
        self.assertTrue(1.31 < log_p < 1.32)

        tpsa = json.loads(utils.tpsa(aspirin))[0]
        self.assertTrue(63 < tpsa < 64)

        descriptors = json.loads(utils.descriptors(aspirin))[0]
        self.assertEqual(descriptors['MolecularFormula'], 'C9H8O4')

    def test_standardise_molecule(self):

        mol = utils.smiles2ctab("[Na]OC(=O)Cc1ccc(C[NH3+])cc1.c1nnn[n-]1.O")
        st = utils.standardise(mol)
        self.assertTrue('V2000' in st)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import re
import sys
import argparse
from chembl_webresource_client.utils import utils
from chembl_webresource_client.new_client import new_client

inchi_key_regex = re.compile('[A-Z]{14}-[A-Z]{10}-[A-Z]')
smilesRegex = re.compile(r'^([^J][.0-9BCGOHMNSEPRIFTLUA@+\-\[\]\(\)\\\/%=#$]+)$')


# ----------------------------------------------------------------------------------------------------------------------


def inspect_synonyms(entry, term):
    return term.lower() in {x['molecule_synonym'].lower() for x in entry['molecule_synonyms']}


# ----------------------------------------------------------------------------------------------------------------------

def get_options():
    description = 'Try to convert names and identifiers into ChEMBL IDs. By default input is read from the standard ' \
                  'input and output written to the standard output'
    parser = argparse.ArgumentParser(description=description, prog='chembl_ids')
    parser.add_argument('-i', '--input', action='store', dest='input', help='input file with names, one line each')
    parser.add_argument('-o', '--output', action='store', dest='output', help='output file with ChEMBL IDs')
    return parser.parse_args()


# ----------------------------------------------------------------------------------------------------------------------


def resolve(mystery):
    ret = 'NOT FOUND'
    if mystery.istartswith('chembl'):
        return mystery.upper()

    molecule = new_client.molecule

    res = molecule.filter(pref_name__iexact=mystery)
    if len(res):
        return ', '.join(x['molecule_chembl_id'] for x in res)

    res = [x for x in molecule.search(mystery) if inspect_synonyms(x, mystery)]
    if len(res):
        return ', '.join(x['molecule_chembl_id'] for x in res)

    if inchi_key_regex.match(mystery.upper()):
        res = molecule.get(mystery.upper())
        if res:
            ret = res['molecule_chembl_id']
        return ret

    if smilesRegex.match(mystery):
        inchi_key = utils.inchi2inchiKey(utils.ctab2inchi(utils.smiles2ctab(mystery)))
    elif mystery.upper().startswith('INCHI='):
        inchi_key = utils.inchi2inchiKey(mystery)
    if inchi_key:
        res = molecule.get(mystery.upper())
    if res:
        ret = res['molecule_chembl_id']
    return ret


# ----------------------------------------------------------------------------------------------------------------------


def main():
    options = get_options()
    with open(options.input) if options.input else sys.stdin as in_f, \
            open(options.output, 'w') if options.output else sys.stdout as out_f:

        out_f.write('\t'.join(('Name:', 'ChEMBL ID:')))

        for line in in_f:
            name = line.strip()
            if not name:
                continue
            resolved = 'NOT FOUND'
            try:
                resolved = resolve(name)
            except:
                pass
            out_f.write('\t'.join((name, resolved)))


# ----------------------------------------------------------------------------------------------------------------------


if __name__ == "__main__":
    main()


# ----------------------------------------------------------------------------------------------------------------------



from chembl_webresource_client.settings import Settings
import os, os.path
import requests
import requests_cache
from requests.packages.urllib3.util import Retry


class Query(object):

    def __init__(self):
        self.session = None
        self._get_session()

    def _get_session(self):
        s = Settings.Instance()
        if not self.session:
            retry = Retry(total=s.TOTAL_RETRIES, backoff_factor=s.BACKOFF_FACTOR,
                          status_forcelist=(list(range(400, 421)) + list(range(500, 505))))
            size = s.CONCURRENT_SIZE
            adapter = requests.adapters.HTTPAdapter(pool_connections=size, pool_maxsize=size,
                                                    pool_block=True, max_retries=retry)
            home_directory = os.path.expanduser('~')
            self.session = requests_cache.CachedSession(
                os.path.join(home_directory, s.CACHE_NAME),
                backend='sqlite',
                expire_after=s.CACHE_EXPIRE,
                fast_save=s.FAST_SAVE,
                allowable_methods=('GET', 'POST'),
                include_get_headers=True) if s.CACHING else requests.Session()
            if s.PROXIES:
                self.session.proxies = s.PROXIES
            self.session.headers.update({"X-HTTP-Method-Override": "GET"})
            self.session.headers.update({'Content-type': 'application/json'})
            self.session.mount('http://', adapter)
            self.session.mount('https://', adapter)
        return self.session



#!/usr/bin/env python
from __future__ import print_function

__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import sys
import argparse
from chembl_webresource_client.scripts.utils import resolve
from chembl_webresource_client.scripts.utils import get_parents
from chembl_webresource_client.scripts.utils import get_serializer

# ----------------------------------------------------------------------------------------------------------------------


def get_options():

    description = 'Try to convert various chemical names and identifiers into a single type of identifiers taken from' \
                  'the ChEMBL DB (by default ChEMBL IDs). By default input is read from the standard input and ' \
                  'output written to the standard output'
    parser = argparse.ArgumentParser(description=description, prog='chembl_ids')
    parser.add_argument('-i', '--input', action='store', dest='input',
                        help='input file, standard input by default')
    parser.add_argument('-o', '--output', action='store', dest='output',
                        help='output file, standard output by default')
    parser.add_argument('-f', '--format', action='store', dest='format', default='chembl_id',
                        help='output file format, can be chosen from 5 options: [chembl_id, smi, sdf, inchi, inchi_key]')
    parser.add_argument('-s', '--single', action='store_true', dest='single',
                        help='if the name is resolved into more than one result, show only the most relevant one')
    parser.add_argument('-p', '--parent', action='store_true', dest='parent',
                        help='instead of actual results, fetch their parents')
    parser.add_argument('-H', '--Human', action='store_true', dest='human',
                        help='human readable output: prints header and first column with original names')
    return parser.parse_args()

# ----------------------------------------------------------------------------------------------------------------------


def main():

    options = get_options()
    with open(options.input) if options.input else sys.stdin as in_f, \
            open(options.output, 'w') if options.output else sys.stdout as out_f:

        serializer_cls = get_serializer(options.format)
        if not serializer_cls:
            sys.stderr.write('Unsupported format', options.format)
            return

        if options.human:
            serializer_cls.write_header(out_f)

        for line in in_f:
            name = line.strip()
            if not name:
                continue
            resolved = None
            try:
                resolved = resolve(name, options.single)
            except Exception as e:
                pass
            if options.parent:
                resolved = get_parents(resolved)
            out_f.write(serializer_cls.serialize_line(resolved, human=options.human, name=name))

# ----------------------------------------------------------------------------------------------------------------------


if __name__ == "__main__":
    main()


# ----------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python
from __future__ import print_function


__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import sys
import argparse
from chembl_webresource_client.scripts.utils import get_serializer, chembl_id_regex, smiles_regex, convert_to_smiles
from chembl_webresource_client.scripts.utils import resolve, mols_to_targets

AVAILABLE_SOURCE_FORMATS = ('chembl_id', 'sdf', 'smi')

# ----------------------------------------------------------------------------------------------------------------------


def get_options():

    description = 'Find related targets for a set of compounds'
    parser = argparse.ArgumentParser(description=description, prog='chembl_m2t')
    parser.add_argument('-i', '--input', action='store', dest='input',
                        help='input file, standard input by default')
    parser.add_argument('-o', '--output', action='store', dest='output',
                        help='output file, standard output by default')
    parser.add_argument('-s', '--source-format', action='store', dest='source_format', default='csv',
                        help='input file format. Can be one of 3: chembl_id (a comma separated list of chembl IDs), '
                             'sdf: (MDL molfile), smi (file containing smiles)')
    parser.add_argument('-d', '--destination-format', action='store', dest='dest_format', default='uniprot',
                        help='output file format. can be chosen from 3 options: '
                             '[uniprot, gene_name, chembl_id]')
    parser.add_argument('-H', '--Human', action='store_true', dest='human',
                        help='human readable output: prints header and first column with original names')
    parser.add_argument('-O', '--organism', action='store', dest='organism',
                        help='Filter results by organism')
    parser.add_argument('-p', '--parent', action='store_true', dest='parent',
                        help='when fetching targets include also targets from parents of given molecules')
    parser.add_argument('-c', '--chunk-size', action='store', dest='chunk', default='1000',
                        help='Size of chunk of data retrieved from API')
    return parser.parse_args()

# ----------------------------------------------------------------------------------------------------------------------


def main():
    options = get_options()

    source_format = options.source_format.lower()
    if source_format not in AVAILABLE_SOURCE_FORMATS:
        sys.stderr.write('Unsupported source format', options.source_format)
        return

    inp = sys.stdin
    if source_format == 'sdf':
        with open(options.input) if options.input else sys.stdin as in_f:
            options.input = None
            inp = convert_to_smiles(in_f)

    with open(options.input) if options.input else inp as in_f, \
            open(options.output, 'w') if options.output else sys.stdout as out_f:

        serializer_cls = get_serializer(options.dest_format)
        if not serializer_cls:
            sys.stderr.write('Unsupported format', options.dest_format)
            return

        if options.human:
            serializer_cls.write_header(out_f)

        for line in in_f:
            if not line or line.lower().startswith('smiles'):
                continue
            chunk = line.strip().split()[0]
            identifiers = chunk.strip().split(',')
            valid_identifiers = list()
            for identifier in identifiers:
                if chembl_id_regex.match(identifier):
                    valid_identifiers.append(identifier)
                elif smiles_regex.match(identifier):
                    valid_identifiers.extend([x['molecule_chembl_id'] for x in resolve(identifier)])
            targets = mols_to_targets(valid_identifiers,
                                      organism=options.organism,
                                      only_ids=(options.dest_format == 'chembl_id'),
                                      include_parents=options.parent,
                                      chunk_size=int(options.chunk))
            out_f.write(serializer_cls.serialize_line(targets, human=options.human, name=','.join(valid_identifiers)))

# ----------------------------------------------------------------------------------------------------------------------


if __name__ == "__main__":
    main()


# ----------------------------------------------------------------------------------------------------------------------



import json
from chembl_webresource_client.query import Query
from chembl_webresource_client.settings import Settings


class ElasticClient(Query):

    def __init__(self):
        super(ElasticClient, self).__init__()

    def _search(self, query, method_name):
        url = '{0}/{1}/_search'.format(Settings.Instance().ELASTIC_URL, method_name)
        res = self.session.post(url, data=
        json.dumps({"size": 0,
             "suggest": {
                 "autocomplete": {
                     "prefix": query,
                     "completion": {
                         "field": "_metadata.es_completion"
                     }
                 }
             }
             }
        ))
        if not res.ok:
            return
        try:
            return [x['_id'] for x in res.json()['suggest']['autocomplete'][0]['options']]
        except:
            pass

    def search_molecule(self, query):
        return self._search(query, 'chembl_molecule')

    def search_target(self, query):
        return self._search(query, 'chembl_target')

    def search_assay(self, query):
        return self._search(query, 'chembl_assay')

    def search_document(self, query):
        return self._search(query, 'chembl_document')

    def search_cell_line(self, query):
        return self._search(query, 'chembl_cell_line')

    def search_tissue(self, query):
        return self._search(query, 'chembl_tissue')




#!/usr/bin/env python
from __future__ import print_function


__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import sys
import argparse
from chembl_webresource_client.scripts.utils import get_serializer
from chembl_webresource_client.scripts.utils import resolve_target, targets_to_mols


# ----------------------------------------------------------------------------------------------------------------------


def get_options():

    description = 'Find related compounds for a set of targets'
    parser = argparse.ArgumentParser(description=description, prog='chembl_t2m')
    parser.add_argument('-i', '--input', action='store', dest='input',
                        help='input file, standard input by default')
    parser.add_argument('-o', '--output', action='store', dest='output',
                        help='output file, standard output by default')
    parser.add_argument('-d', '--destination-format', action='store', dest='dest_format', default='uniprot',
                        help='output file format. can be chosen from 3 options: '
                             '[sdf, smi, chembl_id]')
    parser.add_argument('-H', '--Human', action='store_true', dest='human',
                        help='human readable output: prints header and first column with original names')
    parser.add_argument('-p', '--parent', action='store_true', dest='parent',
                        help='when fetching compounds include their parents as well')
    parser.add_argument('-c', '--chunk-size', action='store', dest='chunk', default='1000',
                        help='Size of chunk of data retrieved from API')
    return parser.parse_args()

# ----------------------------------------------------------------------------------------------------------------------


def main():
    options = get_options()

    with open(options.input) if options.input else sys.stdin as in_f, \
            open(options.output, 'w') if options.output else sys.stdout as out_f:

        serializer_cls = get_serializer(options.dest_format)
        if not serializer_cls:
            sys.stderr.write('Unsupported format', options.dest_format)
            return

        if options.human:
            serializer_cls.write_header(out_f)

        for line in in_f:
            chunk = line.strip().split()[0]
            identifiers = chunk.strip().split(',')
            valid_identifiers = list()
            for identifier in identifiers:
                target = resolve_target(identifier)
                if not target:
                    continue
                valid_identifiers.append(target)
            mols = targets_to_mols(valid_identifiers,
                                   only_ids=(options.dest_format == 'chembl_id'),
                                   include_parents=options.parent,
                                   chunk_size=int(options.chunk))
            out_f.write(serializer_cls.serialize_line(mols, human=options.human, name=','.join(valid_identifiers)))

# ----------------------------------------------------------------------------------------------------------------------


if __name__ == "__main__":
    main()


# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from chembl_webresource_client.settings import Settings

import re
import os
import logging
import requests
import requests_cache
import mimetypes
from chembl_webresource_client.http_errors import handle_http_error
from requests.packages.urllib3.util import Retry

inchi_key_regex = re.compile('[A-Z]{14}-[A-Z]{10}-[A-Z]')

#-----------------------------------------------------------------------------------------------------------------------

mimetypes.init()

#-----------------------------------------------------------------------------------------------------------------------

class UniChemClient(object):

#-----------------------------------------------------------------------------------------------------------------------

    def __init__(self):
        self.session = None
        self._get_session()
        self.base_url = Settings.Instance().UNICHEM_URL
        self.timeout = Settings.Instance().NEW_CLIENT_TIMEOUT
        self.logger = logging.getLogger(__name__)

#-----------------------------------------------------------------------------------------------------------------------

    def _get_session(self):
        s = Settings.Instance()
        if not self.session:
            retry = Retry(total=s.TOTAL_RETRIES, backoff_factor=s.BACKOFF_FACTOR,
                    status_forcelist=(list(range(400, 421)) + list(range(500, 505))))
            size = s.CONCURRENT_SIZE
            adapter = requests.adapters.HTTPAdapter(pool_connections=size, pool_maxsize=size, pool_block=True,
                max_retries=retry)
            home_directory = os.path.expanduser('~')
            self.session = requests_cache.CachedSession(os.path.join(home_directory, s.CACHE_NAME), backend='sqlite',
                expire_after=s.CACHE_EXPIRE, fast_save=s.FAST_SAVE,
                allowable_methods=('GET', 'POST'),
                include_get_headers=True) if s.CACHING else requests.Session()
            if s.PROXIES:
                self.session.proxies = s.PROXIES
            self.session.mount('http://', adapter)
            self.session.mount('https://', adapter)
        return self.session

#-----------------------------------------------------------------------------------------------------------------------

    def _get_results(self, url):
        with self._get_session() as session:
                res = session.get(url, timeout=self.timeout)
                self.logger.info(res.url)
                self.logger.info('From cache: {0}'.format(res.from_cache if hasattr(res, 'from_cache') else False))
                if not res.ok:
                    handle_http_error(res)
                return res.json()

#-----------------------------------------------------------------------------------------------------------------------

    def get(self, pk, src_id=None, to_src_id=None, all=False, url=False, verbose=False):
        if pk.upper().startswith('CHEMBL'):
            if to_src_id:
                if url:
                    url = '{0}/src_compound_id_url/{1}/{2}/{3}'.format(self.base_url, pk, src_id, to_src_id)
                elif all:
                    url = '{0}/src_compound_id_all/{1}/{2}/{3}'.format(self.base_url, pk, src_id, to_src_id)
                elif src_id:
                    url = '{0}/src_compound_id/{1}/{2}/{3}'.format(self.base_url, pk, src_id, to_src_id)
                else:
                    url = '{0}/src_compound_id/{1}/{2}/{3}'.format(self.base_url, pk, 1, to_src_id)
            else:
                if all:
                    url = '{0}/src_compound_id_all/{1}/{2}'.format(self.base_url, pk, src_id)
                elif src_id:
                    url = '{0}/src_compound_id/{1}/{2}'.format(self.base_url, pk, src_id)
                else:
                    url = '{0}/src_compound_id/{1}/{2}'.format(self.base_url, pk, 1)
        elif inchi_key_regex.match(pk):
            if all:
                url = '{0}/inchikey_all/{1}'.format(self.base_url, pk)
            elif verbose:
                url = '{0}/verbose_inchikey/{1}'.format(self.base_url, pk)
            else:
                url = '{0}/inchikey/{1}'.format(self.base_url, pk)
        else:
            if to_src_id:
                url = '{0}/src_compound_id_all_obsolete/{1}/{2}/{3}'.format(self.base_url, pk, src_id, to_src_id)
            elif src_id:
                url = '{0}/src_compound_id_all_obsolete/{1}/{2}'.format(self.base_url, pk, src_id)
            else:
                url = '{0}/orphanIdMap/{1}'.format(self.base_url, pk)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

    def map(self, src, dst=None):
        if dst:
            url = '{0}/mapping/{1}/{2}'.format(self.base_url, src, dst)
        else:
            url = '{0}/mappingaux/{1}'.format(self.base_url, src)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

    def src(self, pk=None):
        if not pk:
            url = '{0}/src_ids/'.format(self.base_url)
        else:
            url = '{0}/sources/{1}'.format(self.base_url, pk)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

    def structure(self,pk,src, all=False):
        if all:
            url = '{0}/structure_all/{1}/{2}'.format(self.base_url,pk,src)
        else:
            url = '{0}/structure/{1}/{2}'.format(self.base_url,pk,src)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

    def inchiFromKey(self, inchi_key):
        url = '{0}/inchi/{1}'.format(self.base_url, inchi_key)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

    def connectivity(self, pk, src_id=None, **kwargs):
        if pk.upper().startswith('CHEMBL'):
            search_type = 'cpd_search'
        else:
            search_type = 'key_search'

        A = kwargs.get('a') or kwargs.get('A') or kwargs.get('sources') or 0
        B = kwargs.get('b') or kwargs.get('B') or kwargs.get('pattern') or 0
        C = kwargs.get('c') or kwargs.get('C') or kwargs.get('component_mapping') or 0
        D = kwargs.get('d') or kwargs.get('D') or kwargs.get('frequency') or 0
        E = kwargs.get('e') or kwargs.get('E') or kwargs.get('inchi_length') or 0
        F = kwargs.get('f') or kwargs.get('F') or kwargs.get('unichem_labels') or 0
        G = kwargs.get('g') or kwargs.get('G') or kwargs.get('assignment_status') or 0
        H = kwargs.get('h') or kwargs.get('H') or kwargs.get('data_structure') or 0

        if B not in (0,1):
            assert False, "Pattern (B) can only be 0 or 1"
        if C not in range(5):
            assert False, "Component mapping (C) can only be in (0,1,2,3,4)"
        if D not in range(501):
            assert False, "Frequency (D) can only be in range 0-500"
        if G not in (0,1):
            assert False, "Assignment status (G) can only be 0 or 1"

        if search_type == 'cpd_search' and src_id:
            url = '{0}/{1}/{2}/{3}/{4}/{5}/{6}/{7}/{8}/{9}/{10}/{11}'.format(
                                                    self.base_url, search_type, pk, src_id, A, B, C, D, E, F, G, H)
        else:
            url = '{0}/{1}/{2}/{3}/{4}/{5}/{6}/{7}/{8}/{9}/{10}'.format(
                                                    self.base_url, search_type, pk, A, B, C, D, E, F, G, H)
        return self._get_results(url)

#-----------------------------------------------------------------------------------------------------------------------

unichem_client = UniChemClient()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'mnowotka'

from setuptools import setup

setup(
    name='chembl-webresource-client',
    version='0.10.9',
    entry_points={
        'console_scripts': [
            'chembl_ids=chembl_webresource_client.scripts.chembl_ids:main',
            'chembl_sim=chembl_webresource_client.scripts.chembl_sim:main',
            'chembl_sub=chembl_webresource_client.scripts.chembl_sub:main',
            'chembl_m2t=chembl_webresource_client.scripts.chembl_m2t:main',
            'chembl_t2m=chembl_webresource_client.scripts.chembl_t2m:main',
            'chembl_act=chembl_webresource_client.scripts.chembl_act:main',
        ]
    },
    author='Michal Nowotka, Eloy Felix',
    author_email='chembl-help@ebi.ac.uk',
    description='Python client fot accessing ChEMBL webservices.',
    url='https://www.ebi.ac.uk/chembl/api/data/docs',
    license='Apache Software License',
    packages=['chembl_webresource_client',
              'chembl_webresource_client.scripts'],
    long_description="""
    Documentation and repository: https://github.com/chembl/chembl_webresource_client.
    This is the only official Python client library developed and supported by ChEMBL (https://www.ebi.ac.uk/chembl/) group.

    The library helps accessing ChEMBL data and cheminformatics tools from Python. 
    You don't need to know how to write SQL. 
    You don't need to know how to interact with REST APIs. 
    You don't need to compile or install any cheminformatics framework. 
    Results are cached.
    """,
    install_requires=[
        'urllib3',
        'requests>=2.18.4',
        'requests-cache~=1.2',
        'easydict',
    ],
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 3.8',
                 'Programming Language :: Python :: 3.9',
                 'Programming Language :: Python :: 3.10',
                 'Programming Language :: Python :: 3.11',
                 'Topic :: Scientific/Engineering :: Chemistry'],
    zip_safe=False,
)



__author__ = 'mnowotka'

from chembl_webresource_client.singleton import Singleton
from chembl_webresource_client import __version__
from pprint import pformat

default_cache_name = '.chembl_ws_client__' + str(__version__)


@Singleton
class Settings:
    WEBSERVICE_PROTOCOL = 'https'
    WEBSERVICE_DOMAIN = 'www.ebi.ac.uk'
    CACHING = True
    FAST_SAVE = True
    TOTAL_RETRIES = 3
    BACKOFF_FACTOR = 2
    CONCURRENT_SIZE = 50
    CACHE_EXPIRE = 60 * 60 * 24
    CACHE_NAME = default_cache_name
    RESPECT_RATE_LIMIT = True
    TIMEOUT = 3.0
    UTILS_SPORE_URL = 'https://www.ebi.ac.uk/chembl/api/utils/spore'
    ELASTIC_URL = 'https://wwwdev.ebi.ac.uk/chembl/glados-es'
    NEW_CLIENT_URL = 'https://www.ebi.ac.uk/chembl/api/data'
    UNICHEM_URL = 'https://www.ebi.ac.uk/unichem/legacy/rest'
    NEW_CLIENT_TIMEOUT = None
    TEST_CASE_TIMEOUT = 10
    MAX_LIMIT = 20
    REPR_OUTPUT_SIZE = 5
    MAX_URL_SIZE = 4000
    PROXIES = None
    CLIENT_VERSION_PICKLE_KEY = 'chembl_webresource_client_version'

    def clear_cache(self):
        from requests_cache import clear
        clear()

    def __str__(self):
        return 'ChEMBL API client settings:\n' + \
               pformat({k: v for k, v in vars(self.__class__).items() if not k.startswith('__') and not callable(v)
                        and not isinstance(v, property) and not k[0].islower()})[1:-1]
